@@ -23,7 +23,9 @@ Module for cloning an existing virtual machine
 General workflow for cloning:
 
     - Instantiate CloneDesign. Requires at least a libvirt connection and
-      a name of a domain to clone.
+      either the name of an existing domain to clone (original_guest), or
+      a string of libvirt xml representing the guest to clone
+      (original_guest_xml)
 
     - Run 'setup' from the CloneDesign instance to prep for cloning
 
@@ -37,6 +39,7 @@ import subprocess
 import _util
 import libvirt
 import Guest
+from VirtualNetworkInterface import VirtualNetworkInterface
 from VirtualDisk import VirtualDisk
 from virtinst import _virtinst as _
 
@@ -47,14 +50,14 @@ class CloneDesign(object):
 
     def __init__(self, connection):
         # hypervisor connection
+        if not isinstance(connection, libvirt.virConnect):
+            raise ValueError(_("Connection must be a 'virConnect' instance."))
         self._hyper_conn = connection
 
         # original guest name or uuid
         self._original_guest        = None
         self._original_dom          = None
-        self._original_devices      = []
-        self._original_devices_size = []
-        self._original_devices_type = []
+        self._original_virtual_disks = []
         self._original_xml          = None
 
         # Deliberately private: user doesn't need to know this
@@ -63,8 +66,7 @@ class CloneDesign(object):
         # clone guest
         self._clone_name         = None
         self._clone_devices      = []
-        self._clone_devices_size = []
-        self._clone_devices_type = []
+        self._clone_virtual_disks = []
         self._clone_bs           = 1024*1024*10
         self._clone_mac          = []
         self._clone_uuid         = None
@@ -78,26 +80,31 @@ class CloneDesign(object):
         # Throwaway guest to use for easy validation
         self._valid_guest        = Guest.Guest(connection=connection)
 
+        # Generate a random UUID at the start
+        while 1:
+            uuid = _util.uuidToString(_util.randomUUID())
+            if _util.vm_uuid_collision(self._hyper_conn, uuid):
+                continue
+            self.clone_uuid = uuid
+            break
 
     # Getter/Setter methods
 
     def get_original_guest(self):
         return self._original_guest
     def set_original_guest(self, original_guest):
-        if type(original_guest) is not type("str") or len(original_guest)==0:
-            raise ValueError, _("Name or UUID of guest to clone is required")
-
-        try:
-            self._valid_guest.set_uuid(original_guest)
-        except ValueError:
-            try:
-                self._valid_guest.set_name(original_guest)
-            except ValueError:
-                raise ValueError, \
-                    _("A valid name or UUID of guest to clone is required")
-        self._original_guest = original_guest
+        if self._lookup_vm(original_guest):
+            self._original_guest = original_guest
     original_guest = property(get_original_guest, set_original_guest)
 
+    def set_original_xml(self, val):
+        if type(val) is not str:
+            raise ValueError(_("Original xml must be a string."))
+        self._original_xml = val
+    def get_original_xml(self):
+        return self._original_xml
+    original_xml = property(get_original_xml, set_original_xml)
+
     def get_clone_name(self):
         return self._clone_name
     def set_clone_name(self, name):
@@ -105,6 +112,15 @@ class CloneDesign(object):
             self._valid_guest.set_name(name)
         except ValueError, e:
             raise ValueError, _("Invalid name for new guest: %s") % (str(e),)
+
+        # Make sure new VM name isn't taken.
+        try:
+            if self._hyper_conn.lookupByName(name) is not None:
+                raise ValueError(_("Domain name '%s' already in use.") %
+                                 name)
+        except libvirt.libvirtError:
+            pass
+
         self._clone_name = name
     clone_name = property(get_clone_name, set_clone_name)
 
@@ -113,30 +129,38 @@ class CloneDesign(object):
             self._valid_guest.set_uuid(uuid)
         except ValueError, e:
             raise ValueError, _("Invalid uuid for new guest: %s") % (str(e),)
+
+        if _util.vm_uuid_collision(self._hyper_conn, uuid):
+            raise ValueError(_("UUID '%s' is in use by another guest.") %
+                             uuid)
         self._clone_uuid = uuid
     def get_clone_uuid(self):
         return self._clone_uuid
     clone_uuid = property(get_clone_uuid, set_clone_uuid)
 
-    def set_clone_devices(self, devices):
+    def set_clone_devices(self, devpath):
         # Devices here is a string path. Every call to set_clone_devices
         # Adds the path (if valid) to the internal _clone_devices list
-        if len(devices) == 0:
-            raise ValueError, _("New file to use for disk image is required")
 
-        # Check path's size (if present)
-        # XXX: Only works locally
-        cdev_size, dummy = self._local_paths_info([devices])
+        # Check path is valid
+        try:
+            disk = VirtualDisk(devpath, size=.0000001, conn=self._hyper_conn)
+        except Exception, e:
+            raise ValueError(_("Could not use path '%s' for cloning: %s") %
+                             (devpath, str(e)))
 
-        # Make sure path is valid and we can use it
-        devices = self._check_file(self._hyper_conn, devices, cdev_size[0])
-        self._clone_devices.append(devices)
+        self._clone_virtual_disks.append(disk)
+        self._clone_devices.append(devpath)
     def get_clone_devices(self):
         return self._clone_devices
     clone_devices = property(get_clone_devices, set_clone_devices)
 
+    def get_clone_virtual_disks(self):
+        return self._clone_virtual_disks
+    clone_virtual_disks = property(get_clone_virtual_disks)
+
     def set_clone_mac(self, mac):
-        Guest.VirtualNetworkInterface(mac, conn=self.original_conn)
+        VirtualNetworkInterface(mac, conn=self.original_conn)
         self._clone_mac.append(mac)
     def get_clone_mac(self):
         return self._clone_mac
@@ -149,13 +173,23 @@ class CloneDesign(object):
     clone_bs = property(get_clone_bs, set_clone_bs)
 
     def get_original_devices_size(self):
-        return self._original_devices_size
+        ret = []
+        for disk in self.original_virtual_disks:
+            ret.append(disk.size)
+        return ret
     original_devices_size = property(get_original_devices_size)
 
     def get_original_devices(self):
-        return self._original_devices
+        ret = []
+        for disk in self.original_virtual_disks:
+            ret.append(disk.path)
+        return ret
     original_devices = property(get_original_devices)
 
+    def get_original_virtual_disks(self):
+        return self._original_virtual_disks
+    original_virtual_disks = property(get_original_virtual_disks)
+
     def get_hyper_conn(self):
         return self._hyper_conn
     original_conn = property(get_hyper_conn)
@@ -164,10 +198,6 @@ class CloneDesign(object):
         return self._original_dom
     original_dom = property(get_original_dom)
 
-    def get_original_xml(self):
-        return self._original_xml
-    original_xml = property(get_original_xml)
-
     def get_clone_xml(self):
         return self._clone_xml
     def set_clone_xml(self, clone_xml):
@@ -201,46 +231,35 @@ class CloneDesign(object):
         """
         logging.debug("Validating original guest parameters")
 
-        try:
-            self._original_dom = self._hyper_conn.lookupByName(self._original_guest)
-        except libvirt.libvirtError:
-            raise RuntimeError, _("Domain %s is not found") % self._original_guest
+        if self.original_guest == None and self.original_xml == None:
+            raise RuntimeError(_("Original guest name or xml is required."))
 
-        # For now, clone_xml is just a copy of the original
-        self._original_xml = self._original_dom.XMLDesc(0)
-        self._clone_xml    = self._original_dom.XMLDesc(0)
+        if self.original_guest != None:
+            self._original_dom = self._lookup_vm(self.original_guest)
+            self.original_xml = self._original_dom.XMLDesc(0)
 
         # Pull clonable storage info from the original xml
-        self._original_devices,     \
-        self._original_devices_size,\
-        self._original_devices_type,\
+        self._original_virtual_disks, \
         self._original_devices_idx = self._get_original_devices_info(self._original_xml)
 
-        logging.debug("Original paths: %s" % (self._original_devices))
-        logging.debug("Original sizes: %s" % (self._original_devices_size))
-        logging.debug("Original types: %s" % (self._original_devices_type))
+        logging.debug("Original paths: %s" % (self.original_devices))
+        logging.debug("Original sizes: %s" % (self.original_devices_size))
         logging.debug("Original idxs: %s" % (self._original_devices_idx))
 
-        # Check original domain is SHUTOFF
-        # XXX: Shouldn't pause also be fine, and guests with no storage/
-        # XXX: readonly + sharable storage can be cloned while running
-        status = self._original_dom.info()[0]
-        logging.debug("original guest status: %s" % (status))
-        if status != libvirt.VIR_DOMAIN_SHUTOFF:
-            raise RuntimeError, _("Domain status must be SHUTOFF")
+        # If there are any devices that need to be cloned and we are on
+        # a remote connection, fail
+        if (self.original_devices and
+            _util.is_uri_remote(self.original_conn.getURI())):
+            raise RuntimeError(_("Cannot clone remote VM storage."))
 
-        # Make sure new VM name isn't taken.
-        # XXX: Check this at set time?
-        try:
-            if self._hyper_conn.lookupByName(self._clone_name) is not None:
-                raise RuntimeError, _("Domain %s already exists") % self._clone_name
-        except libvirt.libvirtError:
-            pass
+        # If domain has devices to clone, it must be 'off' or 'paused'
+        if self._original_dom and len(self.original_devices) != 0:
+            status = self._original_dom.info()[0]
 
-        # Check specified UUID isn't taken
-        if _util.vm_uuid_collision(self._hyper_conn, self._clone_uuid):
-            raise RuntimeError, _("The UUID you entered is already in use by "
-                                  "another guest!")
+            if status not in [libvirt.VIR_DOMAIN_SHUTOFF,
+                              libvirt.VIR_DOMAIN_PAUSED]:
+                raise RuntimeError, _("Domain with devices to clone must be "
+                                      "paused or shutoff.")
 
         # Check mac address is not in use
         # XXX: Check this at set time?
@@ -259,19 +278,13 @@ class CloneDesign(object):
         """
         logging.debug("Validating clone parameters.")
 
+        self._clone_xml = self.original_xml
+
         # XXX: Make sure a clone name has been specified? or generate one?
 
-        # XXX: Only works locally
-        self._clone_devices_size,\
-        self._clone_devices_type = self._local_paths_info(self._clone_devices)
-
         logging.debug("Clone paths: %s" % (self._clone_devices))
-        logging.debug("Clone sizes: %s" % (self._clone_devices_size))
-        logging.debug("Clone types: %s" % (self._clone_devices_type))
 
         # We simply edit the original VM xml in place
-        # XXX: Does this need a huge try except so we don't leak xml memory
-        # XXX: on failure?
         doc = libxml2.parseDoc(self._clone_xml)
         ctx = doc.xpathNewContext()
         typ = ctx.xpathEval("/domain")[0].prop("type")
@@ -288,20 +301,12 @@ class CloneDesign(object):
             try:
                 node.setContent(clone_devices.next())
             except Exception:
-                raise ValueError, _("Missing new file to use disk image "
-                                    "for %s") % node.getContent()
+                raise ValueError, _("Missing path to use as disk clone "
+                                    "destination for '%s'") % node.getContent()
 
-        # changing uuid
+        # We always have a UUID since one is generated at init time
         node = ctx.xpathEval("/domain/uuid")
-        if self._clone_uuid is not None:
-            node[0].setContent(self._clone_uuid)
-        else:
-            while 1:
-                uuid = _util.uuidToString(_util.randomUUID())
-                if _util.vm_uuid_collision(self._hyper_conn, uuid):
-                    continue
-                break
-            node[0].setContent(uuid)
+        node[0].setContent(self._clone_uuid)
 
         # changing mac
         count = ctx.xpathEval("count(/domain/devices/interface/mac)")
@@ -321,10 +326,15 @@ class CloneDesign(object):
 
         # Change xml disk type values if original and clone disk types
         # (block/file) don't match
-        self._change_disk_type(self._original_devices_type,
-                               self._clone_devices_type,
-                               self._original_devices_idx,
-                               ctx)
+        for i in range(0, len(self._original_devices_idx)):
+            orig_type = (self._original_virtual_disks[i].type ==
+                         VirtualDisk.TYPE_FILE)
+            clone_type = (self._clone_virtual_disks[i].type ==
+                          VirtualDisk.TYPE_FILE)
+
+            self._change_disk_type(orig_type, clone_type,
+                                   self._original_devices_idx[i],
+                                   ctx)
 
         # Save altered clone xml
         self._clone_xml = str(doc)
@@ -347,15 +357,9 @@ class CloneDesign(object):
 
     # Private helper functions
 
-    # Check if new file path is valid
-    def _check_file(self, conn, disk, size):
-        d = VirtualDisk(disk, size, conn=conn)
-        return d.path
-
     # Check if new mac address is valid
     def _check_mac(self, mac):
-        nic = Guest.VirtualNetworkInterface(macaddr=mac,
-                                            conn=self.original_conn)
+        nic = VirtualNetworkInterface(macaddr=mac, conn=self.original_conn)
         return nic.is_conflict_net(self._hyper_conn)
 
     # Parse disk paths that need to be cloned from the original guest's xml
@@ -364,9 +368,8 @@ class CloneDesign(object):
     #  [file/block type of those paths], [indices of disks to be cloned])
     def _get_original_devices_info(self, xml):
 
-        lst  = []
-        size = []
-        typ  = []
+        disks   = []
+        lst     = []
         idx_lst = []
 
         doc = libxml2.parseDoc(xml)
@@ -375,7 +378,8 @@ class CloneDesign(object):
             count = ctx.xpathEval("count(/domain/devices/disk)")
             for i in range(1, int(count+1)):
                 # Check if the disk needs cloning
-                node = self._get_available_cloning_device(ctx, i, self._force_target)
+                node = self._get_available_cloning_device(ctx, i,
+                                                          self._force_target)
                 if node == None:
                     continue
                 idx_lst.append(i)
@@ -386,13 +390,24 @@ class CloneDesign(object):
             if doc is not None:
                 doc.freeDoc()
 
-        # Lookup size and storage type (file/block)
-        for i in lst:
-            (t, sz) = _util.stat_disk(i)
-            typ.append(t)
-            size.append(sz)
+        # Set up virtual disk to encapsulate all relevant path info
+        for path in lst:
+            d = None
+            try:
+                try:
+                    d = VirtualDisk(path, conn=self._hyper_conn)
+                except:
+                    # Let's specify a size, and see if the error goes away.
+                    # If so, we know the media doesn't exist
+                    d = VirtualDisk(path, conn=self._hyper_conn, size=.0001)
+                    raise ValueError(_("Disk '%s' does not exist.") %
+                                     d.path)
+            except Exception, e:
+                raise ValueError(_("Could not determine original disk "
+                                   "information: %s" % str(e)))
+            disks.append(d)
 
-        return (lst, size, typ, idx_lst)
+        return (disks, idx_lst)
 
     # Pull disk #i from the original guest xml, return it's xml
     # if it should be cloned (skips readonly, empty, or sharable disks
@@ -425,69 +440,68 @@ class CloneDesign(object):
 
         return node
 
-    # Stat each path in the passed list, return a tuple of
-    # ([size of each path (0 if non-existent)],
-    #  [file/block type of each path (true for 'file', false for 'block')]
-    def _local_paths_info(self, paths_lst):
-
-        size = []
-        typ  = []
-
-        for i in paths_lst:
-            (t, sz) = _util.stat_disk(i)
-            typ.append(t)
-            size.append(sz)
-
-        return (size, typ)
-
     # Check if original disk type (file/block) is different from
     # requested clones disk type, and alter xml if needed
-    def _change_disk_type(self, org_type, cln_type, idxs, ctx):
+    def _change_disk_type(self, org_type, cln_type, dev_idx, ctx):
 
-        type_idx = 0
-        for dev_idx in idxs:
-            disk_type = ctx.xpathEval("/domain/devices/disk[%d]/@type" %
-                                      dev_idx)
-            driv_name = ctx.xpathEval("/domain/devices/disk[%d]/driver/@name" % dev_idx)
-            src = ctx.xpathEval("/domain/devices/disk[%d]/source" % dev_idx)
-            src_chid_txt = src[0].get_properties().getContent()
+        disk_type = ctx.xpathEval("/domain/devices/disk[%d]/@type" % dev_idx)
+        driv_name = ctx.xpathEval("/domain/devices/disk[%d]/driver/@name" %
+                                  dev_idx)
+        src = ctx.xpathEval("/domain/devices/disk[%d]/source" % dev_idx)
+        src_chid_txt = src[0].get_properties().getContent()
 
-            # different type
-            if org_type[type_idx] != cln_type[type_idx]:
-                if org_type[type_idx] == True:
-                    # changing from file to disk
-                    typ, driv, newprop = ("block", "phy", "dev")
-                else:
-                    # changing from disk to file
-                    typ, driv, newprop = ("file", "file", "file")
+        # different type
+        if org_type != cln_type:
+            if org_type == True:
+                # changing from file to disk
+                typ, driv, newprop = ("block", "phy", "dev")
+            else:
+                # changing from disk to file
+                typ, driv, newprop = ("file", "file", "file")
 
-                disk_type[0].setContent(typ)
-                if driv_name:
-                    driv_name[0].setContent(driv)
-                src[0].get_properties().unlinkNode()
-                src[0].newProp(newprop, src_chid_txt)
+            disk_type[0].setContent(typ)
+            if driv_name:
+                driv_name[0].setContent(driv)
+            src[0].get_properties().unlinkNode()
+            src[0].newProp(newprop, src_chid_txt)
+
+
+    # Simple wrapper for checking a vm exists and returning the domain
+    def _lookup_vm(self, name):
+        try:
+            return self._hyper_conn.lookupByName(name)
+        except libvirt.libvirtError:
+            raise ValueError(_("Domain '%s' was not found.") % str(name))
 
-            type_idx += 1
 
-#
-# start duplicate
-# this function clones the virtual machine according to the ClonDesign object
-#
 def start_duplicate(design, meter=None):
+    """
+    Actually perform the duplication: cloning disks if needed and defining
+    the new clone xml.
+    """
 
-    logging.debug("start_duplicate in")
+    logging.debug("Starting duplicate.")
 
-    # do dupulicate
-    # at this point, handling the cloning way.
-    if design.preserve == True:
-        _do_duplicate(design, meter)
+    dom = None
+    try:
+        # Define domain first so we can catch any xml errors before duplicating
+        # storage
+        dom = design.original_conn.defineXML(design.clone_xml)
 
-    # define clone xml
-    design.original_conn.defineXML(design.clone_xml)
+        if design.preserve == True:
+            _do_duplicate(design, meter)
 
-    logging.debug("start_duplicate out")
+    except Exception, e:
+        logging.debug("Duplicate failed: %s" % str(e))
+        if dom:
+            dom.undefine()
+        raise
+
+    logging.debug("Duplicating finished.")
 
 def _vdisk_clone(path, clone):
+    logging.debug("Using vdisk clone.")
+
     path = os.path.expanduser(path)
     clone = os.path.expanduser(clone)
     try:
@@ -496,88 +510,128 @@ def _vdisk_clone(path, clone):
     except OSError:
         return False
 
-#
-# Now this Cloning method is reading and writing devices.
-# For future, there are many cloning methods (e.g. fork snapshot cmd).
-#
+# Iterate over the list of disks, and clone them using the appropriate
+# clone method
 def _do_duplicate(design, meter):
 
-    src_fd = None
-    dst_fd = None
-    dst_dev_iter = iter(design.clone_devices)
+    IS_UNKNOWN = 0
+    IS_LOCAL = 1
+    IS_VDISK = 2
+
+    dst_dev_iter = iter(design.clone_virtual_disks)
+
+    clone_type_dict = {}
+
+    # First loop over the devices, validate we can clone them, and
+    # determine what clone operation to use
+    for src_dev in design.original_virtual_disks:
+        dst_dev = dst_dev_iter.next()
+        src_path = src_dev.path
+        dst_path = dst_dev.path
+
+        clone_type = IS_UNKNOWN
+
+        if (_util.is_vdisk(src_path) or
+            (os.path.exists(dst_path) and _util.is_vdisk(dst_path))):
+
+            if (not _util.is_vdisk(src_path) or
+                os.path.exists(dst_path)):
+                raise RuntimeError, _("copying to an existing vdisk is not"
+                                      " supported")
+            clone_type = IS_VDISK
+
+        else:
+            clone_type = IS_LOCAL
+
+        if clone_type == IS_UNKNOWN:
+            raise RuntimeError(_("Could not determine storage type for '%s'")
+                               % src_path)
+        clone_type_dict[src_path] = clone_type
+
+
+    dst_dev_iter = iter(design.clone_virtual_disks)
     dst_siz_iter = iter(design.original_devices_size)
 
-    zeros            = '\0' * 4096
-    sparse_copy_mode = False
+    # Now actually do the cloning
+    for src_dev in design.original_virtual_disks:
+        src_path = src_dev.path
+        dst_dev = dst_dev_iter.next()
+        dst_path = dst_dev.path
+        dst_siz = int(dst_siz_iter.next() * 1024 * 1024 * 1024)
+
+        clone_type = clone_type_dict[src_path]
+
+        if src_path == "/dev/null":
+            # Not really sure why this check was here, but keeping for compat
+            logging.debug("Source dev was /dev/null. Skipping")
+            continue
+        elif src_path == dst_path:
+            logging.debug("Source and destination are the same. Skipping.")
+            continue
+
+        if meter:
+            meter.start(size=dst_siz,
+                        text=_("Cloning %(srcfile)s") % {'srcfile' : src_path})
+
+        if clone_type == IS_LOCAL:
+            _local_clone(src_path, dst_path, dst_siz, design, meter)
+
+        elif clone_type == IS_VDISK:
+            if not _vdisk_clone(src_path, dst_path):
+                raise RuntimeError, _("failed to clone disk")
+            if meter:
+                meter.end(dst_siz)
+
+
+def _local_clone(src_dev, dst_dev, dst_siz, design, meter=None):
+
+    # if a destination file exists and sparse flg is True,
+    # this priority takes a existing file.
+    if (os.path.exists(dst_dev) == False and design.clone_sparse == True):
+        clone_block_size = 4096
+        sparse = True
+        try:
+            fd = os.open(dst_dev, os.O_WRONLY | os.O_CREAT)
+            os.ftruncate(fd, dst_siz)
+        finally:
+            os.close(fd)
+    else:
+        clone_block_size = 1024*1024*10
+        sparse = False
+
+    logging.debug("Local Cloning %s to %s, sparse=%s, block_size=%s" %
+                  (src_dev, dst_dev, sparse, clone_block_size))
+
+    zeros = '\0' * 4096
 
     try:
-        for src_dev in design.original_devices:
-            dst_dev = dst_dev_iter.next()
-            dst_siz = dst_siz_iter.next()
+        src_fd = os.open(src_dev, os.O_RDONLY)
+        dst_fd = os.open(dst_dev, os.O_WRONLY | os.O_CREAT)
 
-            meter.start(size=dst_siz,
-                        text=_("Cloning from %(src)s to %(dst)s...") % \
-                        {'src' : src_dev, 'dst' : dst_dev})
-
-            if src_dev == "/dev/null" or src_dev == dst_dev:
-                meter.end(dst_siz)
-                continue
-
-            # vdisk specific handlings
-            if _util.is_vdisk(src_dev) or (os.path.exists(dst_dev) and
-                                           _util.is_vdisk(dst_dev)):
-                if not _util.is_vdisk(src_dev) or os.path.exists(dst_dev):
-                    raise RuntimeError, _("copying to an existing vdisk is not supported")
-                if not _vdisk_clone(src_dev, dst_dev):
-                    raise RuntimeError, _("failed to clone disk")
-                meter.end(dst_siz)
-                continue
-
-            #
-            # create sparse file
-            # if a destination file exists and sparse flg is True,
-            # this priority takes a existing file.
-            #
-            if os.path.exists(dst_dev) == False and design.clone_sparse == True:
-                design.clone_bs = 4096
-                sparse_copy_mode = True
-                fd = os.open(dst_dev, os.O_WRONLY | os.O_CREAT)
-                os.ftruncate(fd, dst_siz)
-                os.close(fd)
-            else:
-                design.clone_bs = 1024*1024*10
-                sparse_copy_mode = False
-            logging.debug("dst_dev:%s sparse_copy_mode:%s bs:%d" % (dst_dev,sparse_copy_mode,design.clone_bs))
-
-            src_fd = os.open(src_dev, os.O_RDONLY)
-            dst_fd = os.open(dst_dev, os.O_WRONLY | os.O_CREAT)
-
-            i=0
-            while 1:
-                l = os.read(src_fd, design.clone_bs)
-                s = len(l)
-                if s == 0:
+        i=0
+        while 1:
+            l = os.read(src_fd, clone_block_size)
+            s = len(l)
+            if s == 0:
+                if meter:
                     meter.end(dst_siz)
-                    break
-                # check sequence of zeros
-                if sparse_copy_mode == True and zeros == l:
-                    os.lseek(dst_fd, s, 1)
-                else:
-                    b = os.write(dst_fd, l)
-                    if s != b:
+                break
+            # check sequence of zeros
+            if sparse and zeros == l:
+                os.lseek(dst_fd, s, 1)
+            else:
+                b = os.write(dst_fd, l)
+                if s != b:
+                    if meter:
                         meter.end(i)
-                        break
-                i += s
-                if i < dst_siz:
+                    break
+            i += s
+            if i < dst_siz:
+                if meter:
                     meter.update(i)
 
-            os.close(src_fd)
-            src_fd = None
-            os.close(dst_fd)
-            dst_fd = None
     finally:
         if src_fd is not None:
             os.close(src_fd)
         if dst_fd is not None:
             os.close(dst_fd)
-
