@@ -19,33 +19,29 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import os
 import _util
-import DistroManager
-import logging
-import time
-import platform
+import DistroInstaller
 
 from Guest import Guest
 from VirtualDisk import VirtualDisk
-from virtinst import _virtinst as _
-
 
 class FullVirtGuest(Guest):
 
     def __init__(self, type=None, arch=None, connection=None,
                  hypervisorURI=None, emulator=None, installer=None):
         if not installer:
-            installer = DistroManager.DistroInstaller(type = type, os_type = "hvm")
+            installer = DistroInstaller.DistroInstaller(type = type,
+                                                        os_type = "hvm",
+                                                        conn=connection)
         Guest.__init__(self, type, connection, hypervisorURI, installer)
         self.disknode = "hd"
         self.features = { "acpi": None, "pae":
             _util.is_pae_capable(self.conn), "apic": None }
-        if arch is None:
-            arch = platform.machine()
-        self.arch = arch
 
         self.emulator = emulator
+        if arch:
+            self.arch = arch
+
         self.loader = None
         guest = self._caps.guestForOSType(type=self.installer.os_type,
                                           arch=self.arch)
@@ -58,7 +54,7 @@ class FullVirtGuest(Guest):
         # Fall back to default hardcoding
         if self.emulator is None:
             if self.type == "xen":
-                if os.uname()[4] in ("x86_64"):
+                if self._caps.host.arch in ("x86_64"):
                     self.emulator = "/usr/lib64/xen/bin/qemu-dm"
                 else:
                     self.emulator = "/usr/lib/xen/bin/qemu-dm"
@@ -81,17 +77,13 @@ class FullVirtGuest(Guest):
             features[f] = val
         return features
 
-    def get_os_distro(self):
-        return self._lookup_osdict_key("distro")
-    os_distro = property(get_os_distro)
-
     def _get_input_device(self):
         typ = self._lookup_device_param("input", "type")
         bus = self._lookup_device_param("input", "bus")
         return (typ, bus)
 
     def _get_features_xml(self):
-        ret = "<features>\n"
+        ret = "  <features>\n"
         features = self.os_features()
         if features:
             ret += "    "
@@ -102,22 +94,9 @@ class FullVirtGuest(Guest):
             ret += "\n"
         return ret + "  </features>"
 
-    def _get_osblob(self, install):
-        osblob = self.installer._get_osblob(install, hvm = True,
-            arch = self.arch, loader = self.loader, conn = self.conn)
-        if osblob is None:
-            return None
-
-        clockxml = self._get_clock_xml()
-        if clockxml is not None:
-            return "%s\n  %s\n  %s" % (osblob, self._get_features_xml(), \
-                                       clockxml)
-        else:
-            return "%s\n  %s" % (osblob, self._get_features_xml())
-
     def _get_clock_xml(self):
         val = self._lookup_osdict_key("clock")
-        return """<clock offset="%s"/>""" % val
+        return """  <clock offset="%s"/>""" % val
 
     def _get_device_xml(self, install=True):
         emu_xml = ""
@@ -128,39 +107,6 @@ class FullVirtGuest(Guest):
                 """    <console type='pty'/>\n""" +
                 Guest._get_device_xml(self, install))
 
-    def get_continue_inst(self):
-        return self._lookup_osdict_key("continue")
-
-    def continue_install(self, consolecb, meter, wait=True):
-        cont_xml = self.get_config_xml(disk_boot = True)
-        logging.debug("Continuing guest with:\n%s" % cont_xml)
-        meter.start(size=None, text="Starting domain...")
-
-        # As of libvirt 0.5.1 we can't 'create' over an defined VM.
-        # So, redefine the existing domain (which should be shutoff at
-        # this point), and start it.
-        finalxml = self.domain.XMLDesc(0)
-
-        self.domain = self.conn.defineXML(cont_xml)
-        self.domain.create()
-        self.conn.defineXML(finalxml)
-
-        #self.domain = self.conn.createLinux(install_xml, 0)
-        if self.domain is None:
-            raise RuntimeError, _("Unable to start domain for guest, aborting installation!")
-        meter.end(0)
-
-        self.connect_console(consolecb, wait)
-
-        # ensure there's time for the domain to finish destroying if the
-        # install has finished or the guest crashed
-        if consolecb:
-            time.sleep(1)
-
-        # This should always work, because it'll lookup a config file
-        # for inactive guest, or get the still running install..
-        return self.conn.lookupByName(self.name)
-
     def _get_disk_xml(self, install = True):
         """Get the disk config in the libvirt XML format"""
         ret = ""
