@@ -19,9 +19,8 @@
 # MA 02110-1301 USA.
 
 import os
-import logging
 
-import Guest
+import Installer
 import ImageParser
 import CapabilitiesParser as Cap
 import _util
@@ -32,20 +31,39 @@ class ImageInstallerException(Exception):
     def __init__(self, msg):
         Exception.__init__(self, msg)
 
-class ImageInstaller(Guest.Installer):
+class ImageInstaller(Installer.Installer):
     """Installer for image-based guests"""
-    def __init__(self, image, capabilities, boot_index = None):
-        Guest.Installer.__init__(self)
-        self._capabilities = capabilities
+    def __init__(self, image, capabilities=None, boot_index=None, conn=None):
+        Installer.Installer.__init__(self, conn=conn)
+
+        self._arch = None
         self._image = image
+
+        # Set capabilities
+        if self.conn:
+            self._capabilities = Cap.parse(self.conn.getCapabilities())
+        elif capabilities:
+            if not isinstance(capabilities, Cap.Capabilities):
+                raise ValueError(_("'capabilities' must be a "
+                                   "Capabilities instance."))
+            self._capabilities = capabilities
+        else:
+            raise ValueError(_("'conn' or 'capabilities' must be specified."))
+
+        # Set boot _boot_caps/_boot_parameters
         if boot_index is None:
             self._boot_caps = match_boots(self._capabilities,
                                      self.image.domain.boots)
             if self._boot_caps is None:
-                raise ImageInstallerException(_("Could not find suitable boot descriptor for this host"))
+                raise ImageInstallerException(_("Could not find suitable boot "
+                                                "descriptor for this host"))
         else:
+            if (boot_index < 0 or
+                (boot_index + 1) > len(image.domain.boots)):
+                raise ValueError(_("boot_index out of range."))
             self._boot_caps = image.domain.boots[boot_index]
 
+        # Set up internal caps.guest object
         self._guest = self._capabilities.guestForOSType(self.boot_caps.type,
                                                         self.boot_caps.arch)
         if self._guest is None:
@@ -58,17 +76,14 @@ class ImageInstaller(Guest.Installer):
         self.type = self._domain.hypervisor_type
         self.arch = self._guest.arch
 
+
+    # Custom ImageInstaller methods
+
     def is_hvm(self):
         if self._boot_caps.type == "hvm":
             return True
         return False
 
-    def get_arch(self):
-        return self._arch
-    def set_arch(self, arch):
-        self._arch = arch
-    arch = property(get_arch, set_arch)
-
     def get_image(self):
         return self._image
     image = property(get_image)
@@ -77,28 +92,56 @@ class ImageInstaller(Guest.Installer):
         return self._boot_caps
     boot_caps = property(get_boot_caps)
 
+
+    # General Installer methods
+
     def prepare(self, guest, meter, distro = None):
         self._make_disks(guest)
 
         # Ugly: for PV xen, there's no guest.features, and nothing to toggle
-        if self.type != "xen":
+        if self.os_type != "xen":
             for f in ['pae', 'acpi', 'apic']:
                 if self.boot_caps.features[f] & Cap.FEATURE_ON:
                     guest.features[f] = True
                 elif self.boot_caps.features[f] & Cap.FEATURE_OFF:
                     guest.features[f] = False
 
+    def get_install_xml(self, guest, isinstall):
+
+        kernel = { "kernel" : self.boot_caps.kernel,
+                   "initrd" : self.boot_caps.initrd,
+                   "extrargs" : self.boot_caps.cmdline }
+
+        if self.boot_caps.kernel:
+            isinstall = True
+        else:
+            isinstall = False
+
+        # FYI: self.boot_caps.loader is _not_ analagous to guest loader tag
+
+        return self._get_osblob_helper(guest, isinstall=isinstall,
+                                       kernel=kernel,
+                                       bootdev=self.boot_caps.bootdev)
+
+    def post_install_check(self, guest):
+        return True
+
+
+    # Private methods
+
     def _make_disks(self, guest):
         for m in self.boot_caps.drives:
             p = self._abspath(m.disk.file)
             s = None
             if m.disk.size is not None:
                 s = float(m.disk.size)/1024
+
             # FIXME: This is awkward; the image should be able to express
             # whether the disk is expected to be there or not independently
             # of its classification, especially for user disks
             # FIXME: We ignore the target for the mapping in m.target
-            if m.disk.use == ImageParser.Disk.USE_SYSTEM and not os.path.exists(p):
+            if (m.disk.use == ImageParser.Disk.USE_SYSTEM and
+                not os.path.exists(p)):
                 raise ImageInstallerException(_("System disk %s does not exist")
                                               % p)
             device = VirtualDisk.DEVICE_DISK
@@ -113,41 +156,6 @@ class ImageInstaller(Guest.Installer):
 
             guest._install_disks.append(d)
 
-    def _get_osblob(self, install, hvm, arch=None, loader=None, conn=None):
-
-        kernel = { "kernel" : self.boot_caps.kernel,
-                   "initrd" : self.boot_caps.initrd,
-                   "extrargs" : self.boot_caps.cmdline }
-
-        if self.boot_caps.kernel:
-            install = True
-        else:
-            install = False
-
-        # XXX: The installer/guest dichotomy here is kind of messed up,
-        # XXX: since the installer has the 'image' object which contains
-        # XXX: a lot of info that the guest does as well. Take the installers
-        # XXX: data over the guests.
-        arch   = self.boot_caps.arch or arch
-
-        # self.boot_caps.loader is _not_ analagous to guest loader tag
-        loader = loader
-        ishvm  = self.boot_caps.type == "hvm"
-
-        if ishvm != hvm:
-            logging.debug("Guest and ImageInstaller do not agree on virt "
-                          "type. Using Guests'.")
-            ishvm = hvm
-
-        return self._get_osblob_helper(isinstall=install, ishvm=ishvm,
-                                       arch=arch, loader=loader,
-                                       conn=conn, kernel=kernel,
-                                       bootdev=self.boot_caps.bootdev)
-
-
-    def post_install_check(self, guest):
-        return True
-
     def _abspath(self, p):
         return self.image.abspath(p)
 
