@@ -24,15 +24,130 @@ import os
 import gzip
 import re
 import tempfile
-import platform
 import socket
 import ConfigParser
 
+import virtinst
 from virtinst import _util
 from virtinst import _virtinst as _
 
-def distroFromTreeinfo(fetcher, progresscb, uri, vmtype=None,
-                       scratchdir=None, arch=None):
+from ImageFetcher import MountedImageFetcher
+from ImageFetcher import FTPImageFetcher
+from ImageFetcher import HTTPImageFetcher
+from ImageFetcher import DirectImageFetcher
+
+def _fetcherForURI(uri, scratchdir=None):
+    if uri.startswith("http://"):
+        fclass = HTTPImageFetcher
+    elif uri.startswith("ftp://"):
+        fclass = FTPImageFetcher
+    elif uri.startswith("nfs://"):
+        fclass = MountedImageFetcher
+    else:
+        if os.path.isdir(uri):
+            fclass = DirectImageFetcher
+        else:
+            fclass = MountedImageFetcher
+    return fclass(uri, scratchdir)
+
+def _storeForDistro(fetcher, baseuri, typ, progresscb, arch, distro=None,
+                    scratchdir=None):
+    stores = []
+    skip_treeinfo = False
+    logging.debug("Attempting to detect distro:")
+
+    dist = virtinst.OSDistro.distroFromTreeinfo(fetcher, progresscb, baseuri,
+                                                arch, typ, scratchdir)
+    if dist:
+        return dist
+    skip_treeinfo = True
+
+    # FIXME: This 'distro ==' doesn't cut it. 'distro' is from our os
+    # dictionary, so would look like 'fedora9' or 'rhel5', so this needs
+    # to be a bit more intelligent
+    if distro == "fedora" or distro is None:
+        stores.append(FedoraDistro)
+    if distro == "rhel" or distro is None:
+        stores.append(RHELDistro)
+    if distro == "centos" or distro is None:
+        stores.append(CentOSDistro)
+    if distro == "sl" or distro is None:
+        stores.append(SLDistro)
+    if distro == "suse" or distro is None:
+        stores.append(SuseDistro)
+    if distro == "debian" or distro is None:
+        stores.append(DebianDistro)
+    if distro == "ubuntu" or distro is None:
+        stores.append(UbuntuDistro)
+    if distro == "mandriva" or distro is None:
+        stores.append(MandrivaDistro)
+    # XXX: this is really "nevada"
+    if distro == "solaris" or distro is None:
+        stores.append(SolarisDistro)
+    if distro == "solaris" or distro is None:
+        stores.append(OpenSolarisDistro)
+
+    stores.append(GenericDistro)
+
+    for sclass in stores:
+        store = sclass(baseuri, arch, typ, scratchdir)
+        if skip_treeinfo:
+            store.uses_treeinfo = False
+        if store.isValidStore(fetcher, progresscb):
+            return store
+
+    raise ValueError, _("Could not find an installable distribution at '%s'" %
+                        baseuri)
+
+def _acquireMedia(iskernel, guest, baseuri, progresscb, arch,
+                  scratchdir="/var/tmp", _type=None, distro=None):
+    fetcher = _fetcherForURI(baseuri, scratchdir)
+
+    try:
+        fetcher.prepareLocation()
+    except ValueError, e:
+        raise ValueError, _("Invalid install location: ") + str(e)
+
+    try:
+        store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, typ=_type,
+                                progresscb=progresscb, distro=distro,
+                                scratchdir=scratchdir, arch=arch)
+
+        if iskernel is True:
+            # FIXME: We should probably do this for both kernel and boot
+            # disk?
+            return (store.acquireKernel(guest, fetcher, progresscb),
+                    store.os_type, store.os_variant)
+        elif iskernel is False:
+            return store.acquireBootDisk(fetcher, progresscb)
+        else:
+            # Kind of a hack. Just return the store for detectDistro
+            return store
+    finally:
+        fetcher.cleanupLocation()
+
+# Helper method to lookup install media distro and fetch an install kernel
+def acquireKernel(guest, baseuri, progresscb, arch,
+                  scratchdir="/var/tmp", type=None, distro=None):
+    return _acquireMedia(True, guest, baseuri, progresscb, arch,
+                         scratchdir, type, distro)
+
+# Helper method to lookup install media distro and fetch a boot iso
+def acquireBootDisk(baseuri, progresscb, arch, scratchdir="/var/tmp",
+                    type=None, distro=None):
+    return _acquireMedia(False, None, baseuri, progresscb, arch,
+                         scratchdir, type, distro)
+
+# Attempt to detect the os type + variant for the passed location
+def detectMediaDistro(location, arch):
+    import urlgrabber
+    progress = urlgrabber.progress.BaseMeter()
+    store = _acquireMedia(None, None, location, progress, arch, "/var/tmp")
+    return (store.os_type, store.os_variant)
+
+
+def distroFromTreeinfo(fetcher, progresscb, uri, arch, vmtype=None,
+                       scratchdir=None):
     # Parse treeinfo 'family' field, and return the associated Distro class
     # None if no treeinfo, GenericDistro if unknown family type.
     if not fetcher.hasFile(".treeinfo"):
@@ -48,17 +163,22 @@ def distroFromTreeinfo(fetcher, progresscb, uri, vmtype=None,
     fam = treeinfo.get("general", "family")
 
     if re.match(".*Fedora.*", fam):
-        ob = FedoraDistro(uri, vmtype, scratchdir, arch)
+        dclass = FedoraDistro
     elif re.match(".*CentOS.*", fam):
-        ob = CentOSDistro(uri, vmtype, scratchdir, arch)
+        dclass = CentOSDistro
     elif re.match(".*Red Hat Enterprise Linux.*", fam):
-        ob = RHELDistro(uri, vmtype, scratchdir, arch)
+        dclass = RHELDistro
     elif re.match(".*Scientific Linux.*", fam):
-        ob = SLDistro(uri, vmtype, scratchdir, arch)
+        dclass = SLDistro
     else:
-        ob = GenericDistro(uri, vmtype, scratchdir, arch)
+        dclass = GenericDistro
 
+    ob = dclass(uri, arch, vmtype, scratchdir)
     ob.treeinfo = treeinfo
+
+    # Explictly call this, so we populate os_type/variant info
+    ob.isValidStore(fetcher, progresscb)
+
     return ob
 
 
@@ -67,17 +187,20 @@ def distroFromTreeinfo(fetcher, progresscb, uri, vmtype=None,
 class Distro:
 
     name = ""
+
+    # osdict type and variant values
+    os_type = None
+    os_variant = None
+
     _boot_iso_paths = []
     _hvm_kernel_paths = []
     _xen_kernel_paths = []
     uses_treeinfo = False
 
-    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
+    def __init__(self, uri, arch, vmtype=None, scratchdir=None):
         self.uri = uri
         self.type = vmtype
         self.scratchdir = scratchdir
-        if arch == None:
-            arch = platform.machine()
         self.arch = arch
         self.treeinfo = None
 
@@ -121,7 +244,6 @@ class Distro:
             raise RuntimeError(_("Could not find boot.iso in %s tree." % \
                                self.name))
 
-
     def _hasTreeinfo(self, fetcher, progresscb):
         # all Red Hat based distros should have .treeinfo, perhaps others
         # will in time
@@ -174,7 +296,8 @@ class Distro:
 
         return False
 
-    def _kernelFetchHelper(self, fetcher, guest, progresscb, kernelpath, initrdpath):
+    def _kernelFetchHelper(self, fetcher, guest, progresscb, kernelpath,
+                           initrdpath):
         # Simple helper for fetching kernel + initrd and performing
         # cleanup if neccessary
         kernel = fetcher.acquireFile(kernelpath, progresscb)
@@ -184,7 +307,7 @@ class Distro:
             args += "method=" + fetcher.location
 
         if guest.extraargs:
-            args += guest.extraargs
+            args += " " + guest.extraargs
 
         try:
             initrd = fetcher.acquireFile(initrdpath, progresscb)
@@ -277,6 +400,7 @@ class RedHatDistro(Distro):
 
     name = "Red Hat"
     os_type = "linux"
+
     uses_treeinfo = True
     _boot_iso_paths   = [ "images/boot.iso" ]
     _hvm_kernel_paths = [ ("images/pxeboot/vmlinuz",
@@ -292,51 +416,89 @@ class RedHatDistro(Distro):
 class FedoraDistro(RedHatDistro):
 
     name = "Fedora"
-    os_type = "linux"
 
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Fedora.*", self.treeinfo.get("general", "family"))
-            return (m != None)
+            ret = (m != None)
+
+            if ret:
+                ver = self.treeinfo.get("general", "version")
+                if ver == "development":
+                    self.os_variant = self._latestFedoraVariant()
+                elif ver:
+                    self.os_variant = "fedora" + str(ver)
+
+            return ret
         else:
             if fetcher.hasFile("Fedora"):
                 logging.debug("Detected a Fedora distro")
                 return True
             return False
 
+    def _latestFedoraVariant(self):
+        import osdict
+        ret = None
+        for var in osdict.sort_helper(osdict.OS_TYPES["linux"]["variants"]):
+            if var.startswith("fedora"):
+                # Last fedora* occurence should be the newest
+                ret = var
+        return ret
+
 # Red Hat Enterprise Linux distro check
 class RHELDistro(RedHatDistro):
 
     name = "Red Hat Enterprise Linux"
-    os_type = "linux"
 
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
-            m = re.match(".*Red Hat Enterprise Linux.*", self.treeinfo.get("general", "family"))
-            return (m != None)
+            m = re.match(".*Red Hat Enterprise Linux.*",
+                         self.treeinfo.get("general", "family"))
+            ret = (m != None)
+
+            if ret:
+                self._variantFromVersion()
+            return ret
         else:
             # fall back to old code
             if fetcher.hasFile("Server"):
                 logging.debug("Detected a RHEL 5 Server distro")
+                self.os_variant = "rhel5"
                 return True
             if fetcher.hasFile("Client"):
                 logging.debug("Detected a RHEL 5 Client distro")
+                self.os_variant = "rhel5"
                 return True
             if fetcher.hasFile("RedHat"):
                 logging.debug("Detected a RHEL 4 distro")
+                self.os_variant = "rhel4"
                 return True
             return False
 
+    def _variantFromVersion(self):
+        # XXX: Version will look like '5.2', rather than just '5'. We
+        # need to support this in the os dictionary at some point, but for
+        # now, just use the '5' part
+        ver = self.treeinfo.get("general", "version")
+        if not ver:
+            return
+
+        self.os_variant = "rhel" + ver[0]
+
+
 # CentOS distro check
-class CentOSDistro(RedHatDistro):
+class CentOSDistro(RHELDistro):
 
     name = "CentOS"
-    os_type = "linux"
 
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*CentOS.*", self.treeinfo.get("general", "family"))
-            return (m != None)
+            ret = (m != None)
+
+            if ret:
+                self._variantFromVersion()
+            return ret
         else:
             # fall back to old code
             if fetcher.hasFile("CentOS"):
@@ -345,12 +507,12 @@ class CentOSDistro(RedHatDistro):
             return False
 
 # Scientific Linux distro check
-class SLDistro(RedHatDistro):
+class SLDistro(RHELDistro):
 
     name = "Scientific Linux"
-    os_type = "linux"
-    _boot_iso_paths = RedHatDistro._boot_iso_paths + [ "images/SL/boot.iso" ]
-    _hvm_kernel_paths = RedHatDistro._hvm_kernel_paths + \
+
+    _boot_iso_paths = RHELDistro._boot_iso_paths + [ "images/SL/boot.iso" ]
+    _hvm_kernel_paths = RHELDistro._hvm_kernel_paths + \
                         [ ("images/SL/pxeboot/vmlinuz",
                            "images/SL/pxeboot/initrd.img") ]
 
@@ -358,7 +520,11 @@ class SLDistro(RedHatDistro):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Scientific Linux.*",
                          self.treeinfo.get("general", "family"))
-            return (m != None)
+            ret = (m != None)
+
+            if ret:
+                self._variantFromVersion()
+            return ret
         else:
             if fetcher.hasFile("SL"):
                 logging.debug("Detected a Scientific Linux distro")
@@ -377,8 +543,8 @@ class SuseDistro(Distro):
     _hvm_kernel_paths = []
     _xen_kernel_paths = []
 
-    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
-        Distro.__init__(self, uri, vmtype, scratchdir, arch)
+    def __init__(self, arch, uri, vmtype=None, scratchdir=None):
+        Distro.__init__(self, arch, uri, vmtype, scratchdir)
         if re.match(r'i[4-9]86', arch):
             self.arch = 'i386'
 
@@ -604,8 +770,8 @@ class DebianDistro(Distro):
     name = "Debian"
     os_type = "linux"
 
-    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
-        Distro.__init__(self, uri, vmtype, scratchdir, arch)
+    def __init__(self, uri, arch, vmtype=None, scratchdir=None):
+        Distro.__init__(self, arch, uri, vmtype, scratchdir)
         if uri.count("installer-i386"):
             self._treeArch = "i386"
         elif uri.count("installer-amd64"):
@@ -655,7 +821,6 @@ class DebianDistro(Distro):
 class UbuntuDistro(DebianDistro):
 
     name = "Ubuntu"
-    os_type = "linux"
 
     def _set_media_paths(self):
         DebianDistro._set_media_paths(self)
@@ -700,6 +865,9 @@ class MandrivaDistro(Distro):
         # Mandriva websites / media appear to have a VERSION
         # file in top level which we can use as our 'magic'
         # check for validity
+        if not fetcher.hasFile("VERSION"):
+            return False
+
         if self._fetchAndMatchRegex(fetcher, progresscb, "VERSION",
                                     ".*Mandriva.*"):
             logging.debug("Detected a Mandriva distro")
@@ -779,7 +947,7 @@ class SolarisDistro(SunDistro):
            and args, and '-B' boot properties."""
 
         # XXX: ignoring smfargs for the time being
-        (kopts, kargs, smfargs, kbargs) = \
+        (kopts, kargs, ignore_smfargs, kbargs) = \
             self.process_extra_args(guest.extraargs)
 
         args = [ '' ]
@@ -869,7 +1037,7 @@ class OpenSolarisDistro(SunDistro):
            and args, and '-B' boot properties."""
 
         # XXX: ignoring smfargs and kargs for the time being
-        (kopts, kargs, smfargs, kbargs) = \
+        (kopts, ignore_kargs, ignore_smfargs, kbargs) = \
             self.process_extra_args(guest.extraargs)
 
         args = ''
