@@ -20,27 +20,20 @@
 # MA 02110-1301 USA.
 
 from Guest import Guest
-from DistroManager import DistroInstaller
-from virtinst import _virtinst as _
+from DistroInstaller import DistroInstaller
 
 class ParaVirtGuest(Guest):
-    def __init__(self, type=None, connection=None, hypervisorURI=None, installer=None):
+    def __init__(self, type=None, connection=None, hypervisorURI=None,
+                 installer=None):
         if not installer:
-            installer = DistroInstaller(type = type, os_type = "linux")
+            installer = DistroInstaller(type = type, os_type = "xen",
+                                        conn=connection)
         Guest.__init__(self, type, connection, hypervisorURI, installer)
         self.disknode = "xvd"
 
-    def _get_osblob(self, install):
-        return self.installer._get_osblob(install, hvm = False, conn = self.conn)
-
     def _get_input_device(self):
         return ("mouse", "xen")
 
-    def validate_parms(self):
-        if not self.location and not self.boot:
-            raise ValueError, _("A location must be specified to install from")
-        Guest.validate_parms(self)
-
     def _get_disk_xml(self, install = True):
         """Get the disk config in the libvirt XML format"""
         ret = ""
