@@ -48,7 +48,6 @@ General workflow for the different storage objects:
 
 import libvirt
 
-import re
 import logging
 from xml.sax.saxutils import escape
 
@@ -86,6 +85,7 @@ class StorageObject(object):
             raise ValueError, _("Unknown storage object type: %s") % type
         self._object_type = object_type
         self._conn = None
+        self._name = None
         if conn is not None:
             self.conn = conn
 
@@ -117,15 +117,7 @@ class StorageObject(object):
     def get_name(self):
         return self._name
     def set_name(self, val):
-        if type(val) is not type("string") or len(val) > 50 or len(val) == 0:
-            raise ValueError, _("Storage object name must be a string " +
-                                "between 0 and 50 characters")
-        if re.match("^[0-9]+$", val):
-            raise ValueError, _("Storage object name can not be only " +
-                                "numeric characters")
-        if re.match("^[a-zA-Z0-9._-]+$", val) == None:
-            raise ValueError, _("Storage object name can only contain " +
-                                "alphanumeric, '_', '.', or '-' characters")
+        _util.validate_name(_("Storage object"), val)
 
         # Check that name doesn't collide with other storage objects
         self._check_name_collision(val)
@@ -262,6 +254,11 @@ class StoragePool(StorageObject):
         if type not in self.get_pool_types():
             raise ValueError, _("Unknown storage pool type: %s" % type)
         self._type = type
+        self._target_path = None
+        self._host = None
+        self._format = None
+        self._source_path = None
+
         if target_path is None:
             target_path = self._get_default_target_path()
         self.target_path = target_path
@@ -700,12 +697,16 @@ class StorageVolume(StorageObject):
                 raise ValueError(_("'conn' must be specified with 'pool_name'"))
             pool = StorageVolume.lookup_pool_by_name(pool_name=pool_name,
                                                      conn=conn)
+        self._pool = None
+
         self.pool = pool
 
         StorageObject.__init__(self, object_type=StorageObject.TYPE_VOLUME,
                                name=name, conn=self.pool._conn)
         self._allocation = None
         self._capacity = None
+        self._format = None
+
         self.allocation = allocation
         self.capacity = capacity
 
