@@ -19,9 +19,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
-import os, stat, statvfs
+import os, statvfs
 import subprocess
-import libxml2
 import logging
 import libvirt
 
@@ -87,6 +86,12 @@ class VirtualDisk(VirtualDevice):
     driver_types = [DRIVER_TAP_RAW, DRIVER_TAP_QCOW,
         DRIVER_TAP_VMDK, DRIVER_TAP_VDISK]
 
+    CACHE_MODE_NONE = "none"
+    CACHE_MODE_WRITETHROUGH = "writethrough"
+    CACHE_MODE_WRITEBACK = "writeback"
+    cache_types = [CACHE_MODE_NONE, CACHE_MODE_WRITETHROUGH,
+        CACHE_MODE_WRITEBACK]
+
     DEVICE_DISK = "disk"
     DEVICE_CDROM = "cdrom"
     DEVICE_FLOPPY = "floppy"
@@ -99,7 +104,8 @@ class VirtualDisk(VirtualDevice):
     def __init__(self, path=None, size=None, transient=False, type=None,
                  device=DEVICE_DISK, driverName=None, driverType=None,
                  readOnly=False, sparse=True, conn=None, volObject=None,
-                 volInstall=None, volName=None, bus=None):
+                 volInstall=None, volName=None, bus=None, shareable=False,
+                 driverCache=None):
         """
         @param path: filesystem path to the disk image.
         @type path: C{str}
@@ -130,6 +136,10 @@ class VirtualDisk(VirtualDevice):
         @type volName: C{tuple} of (C{str}, C{str})
         @param bus: Emulated bus type (ide, scsi, virtio, ...)
         @type bus: C{str}
+        @param shareable: If disk can be shared among VMs
+        @type shareable: C{bool}
+        @param driverCache: Disk cache mode (none, writethrough, writeback)
+        @type driverCache: member of cache_types
         """
 
         VirtualDevice.__init__(self, conn=conn)
@@ -143,6 +153,8 @@ class VirtualDisk(VirtualDevice):
         self._vol_object = None
         self._vol_install = None
         self._bus = None
+        self._shareable = None
+        self._driver_cache = None
 
         # XXX: No property methods for these
         self.transient = transient
@@ -159,6 +171,8 @@ class VirtualDisk(VirtualDevice):
         self._set_vol_object(volObject, validate=False)
         self._set_vol_install(volInstall, validate=False)
         self._set_bus(bus, validate=False)
+        self._set_shareable(shareable, validate=False)
+        self._set_driver_cache(driverCache, validate=False)
 
         if volName:
             self.__lookup_vol_name(volName)
@@ -262,6 +276,23 @@ class VirtualDisk(VirtualDevice):
         self.__validate_wrapper("_bus", val, validate)
     bus = property(_get_bus, _set_bus)
 
+    def _get_shareable(self):
+        return self._shareable
+    def _set_shareable(self, val, validate=True):
+        self._check_bool(val, "shareable")
+        self.__validate_wrapper("_shareable", val, validate)
+    shareable = property(_get_shareable, _set_shareable)
+
+    def _get_driver_cache(self):
+        return self._driver_cache
+    def _set_driver_cache(self, val, validate=True):
+        if val is not None:
+            self._check_str(val, "cache")
+            if val not in self.cache_types:
+                raise ValueError, _("Unknown cache mode '%s'" % val)
+        self.__validate_wrapper("_driver_cache", val, validate)
+    driver_cache = property(_get_driver_cache, _set_driver_cache)
+
     # Validation assistance methods
 
     # Initializes attribute if it hasn't been done, then validates args.
@@ -279,6 +310,32 @@ class VirtualDisk(VirtualDevice):
                 setattr(self, varname, orig)
                 raise
 
+    def __set_size(self, creating_storage):
+        """
+        Fill in 'size' attribute for existing storage.
+        """
+
+        if creating_storage:
+            return
+
+        if self.__storage_specified() and self.vol_object:
+            newsize = _util.get_xml_path(self.vol_object.XMLDesc(0),
+                                         "/volume/capacity")
+            try:
+                newsize = float(newsize) / 1024.0 / 1024.0 / 1024.0
+            except:
+                newsize = 0
+        elif self.path is None:
+            newsize = 0
+        else:
+            ignore, newsize = _util.stat_disk(self.path)
+            newsize = newsize / 1024.0 / 1024.0 / 1024.0
+
+        if newsize != self.size:
+            logging.debug("Setting size for existing storage to '%s'" %
+                          newsize)
+            self._set_size(newsize, validate=False)
+
     def __set_dev_type(self):
         """
         Detect disk 'type' () from passed storage parameters
@@ -301,10 +358,10 @@ class VirtualDisk(VirtualDevice):
                 # All others should be using TYPE_BLOCK (hopefully)
                 dtype = self.TYPE_BLOCK
         elif self.path:
-            if stat.S_ISBLK(os.stat(self.path)[stat.ST_MODE]):
-                dtype = self.TYPE_BLOCK
-            else:
+            if _util.stat_disk(self.path)[0]:
                 dtype = self.TYPE_FILE
+            else:
+                dtype = self.TYPE_BLOCK
             if _util.is_vdisk(self.path):
                 self._driverName = self.DRIVER_TAP
                 self._driverType = self.DRIVER_TAP_VDISK
@@ -396,8 +453,18 @@ class VirtualDisk(VirtualDevice):
                                capacity=cap, allocation=alloc, pool=pool)
                 self._set_vol_install(vol, validate=False)
             elif self._is_remote():
-                raise ValueError(_("'%s' is not managed on remote "
-                                   "host: %s" % (self.path, verr)))
+
+                if not verr:
+                    # Since there is no error, no pool was ever found
+                    err = (_("Cannot use storage '%(path)s': '%(rootdir)s' is "
+                             "not managed on the remote host.") %
+                             { 'path' : self.path,
+                               'rootdir' : os.path.dirname(self.path)})
+                else:
+                    err = (_("Cannot use storage %(path)s: %(err)s") %
+                           { 'path' : self.path, 'err' : verr })
+
+                raise ValueError(err)
         else:
             self._set_vol_object(vol, validate=False)
 
@@ -412,19 +479,20 @@ class VirtualDisk(VirtualDevice):
         if self.vol_object:
             newpath = self.vol_object.path()
         elif self.vol_install:
-            newpath = _util.get_xml_path(self.vol_install.pool.XMLDesc(0),
-                                         "/pool/target/path") + \
-                      self.vol_install.name
+            newpath = (_util.get_xml_path(self.vol_install.pool.XMLDesc(0),
+                                         "/pool/target/path") + "/" +
+                       self.vol_install.name)
 
         if newpath and newpath != self.path:
-            logging.debug("Overwriting 'path' from passed volume object.")
+            logging.debug("Overwriting 'path' with value from StorageVolume"
+                          " object.")
             self._set_path(newpath, validate=False)
 
         if self.vol_install:
             newsize = self.vol_install.capacity/1024.0/1024.0/1024.0
             if self.size != newsize:
                 logging.debug("Overwriting 'size' with value from "
-                              "StorageVolume")
+                              "StorageVolume object")
                 self._set_size(newsize, validate=False)
 
         # Remove this piece when storage volume creation is async
@@ -475,6 +543,8 @@ class VirtualDisk(VirtualDevice):
         create_media = not ((managed_storage and self.vol_object) or \
                             (self.path and os.path.exists(self.path)))
 
+        self.__set_size(create_media)
+
         if self._is_remote() and not managed_storage:
             raise ValueError, _("Must specify libvirt managed storage if on "
                                 "a remote connection")
@@ -504,12 +574,15 @@ class VirtualDisk(VirtualDevice):
             self.set_type(self.TYPE_FILE, validate=False)
 
             # Path doesn't exist: make sure we have write access to dir
-            if not os.access(os.path.dirname(self.path), os.W_OK):
-                raise ValueError, _("No write access to directory '%s'") % \
-                                    os.path.dirname(self.path)
+            if not os.access(os.path.dirname(self.path), os.R_OK):
+                raise ValueError("No read access to directory '%s'" %
+                                 os.path.dirname(self.path))
             if self.size is None:
                 raise ValueError, _("size is required for non-existent disk "
                                     "'%s'" % self.path)
+            if not os.access(os.path.dirname(self.path), os.W_OK):
+                raise ValueError, _("No write access to directory '%s'") % \
+                                    os.path.dirname(self.path)
         else:
             self.__set_dev_type()
 
@@ -603,26 +676,38 @@ class VirtualDisk(VirtualDevice):
         if path:
             path = _util.xml_escape(path)
 
-        ret = "    <disk type='%(type)s' device='%(device)s'>\n" % { "type": self.type, "device": self.device }
-        if not(self.driver_name is None):
-            if self.driver_type is None:
-                ret += "      <driver name='%(name)s'/>\n" % { "name": self.driver_name }
-            else:
-                ret += "      <driver name='%(name)s' type='%(type)s'/>\n" % { "name": self.driver_name, "type": self.driver_type }
+        ret = "    <disk type='%s' device='%s'>\n" % (self.type, self.device)
+
+        dname = self.driver_name
+        if not dname and self.driver_cache:
+            self.driver_name = "qemu"
+
+        if not self.driver_name is None:
+            dtypexml = ""
+            if not self.driver_type is None:
+                dtypexml = " type='%s'" % self.driver_type
+
+            dcachexml = ""
+            if not self.driver_cache is None:
+                dcachexml = " cache='%s'" % self.driver_cache
+
+            ret += "      <driver name='%s'%s%s/>\n" % (self.driver_name,
+                                                      dtypexml, dcachexml)
+
         if path is not None:
-            ret += "      <source %(typeattr)s='%(disk)s'/>\n" % { "typeattr": typeattr, "disk": path }
+            ret += "      <source %s='%s'/>\n" % (typeattr, path)
 
         bus_xml = ""
         if self.bus is not None:
             bus_xml = " bus='%s'" % self.bus
-        ret += "      <target dev='%s'" % disknode + \
-                      "%s" % bus_xml + \
-                      "/>\n"
+        ret += "      <target dev='%s'%s/>\n" % (disknode, bus_xml)
 
         ro = self.read_only
 
         if self.device == self.DEVICE_CDROM:
             ro = True
+        if self.shareable:
+            ret += "      <shareable/>\n"
         if ro:
             ret += "      <readonly/>\n"
         ret += "    </disk>"
@@ -666,36 +751,23 @@ class VirtualDisk(VirtualDevice):
                         ((need / (1024*1024)), (avail / (1024*1024)))
         return (ret, msg)
 
-    def is_conflict_disk(self, conn):
+    def is_conflict_disk(self, conn, return_names=False):
         """
         check if specified storage is in use by any other VMs on passed
         connection.
 
         @param conn: connection to check for collisions on
         @type conn: libvirt.virConnect
+        @param return_names: Whether or not to return a list of VM names using
+                             the same storage (default = False)
+        @type return_names: C{bool}
 
-        @return: True if a collision, False otherwise
+        @return: True if a collision, False otherwise (list of names if
+                 return_names passed)
         @rtype: C{bool}
         """
-        vms = []
-        # get working domain's name
-        ids = conn.listDomainsID()
-        for i in ids:
-            try:
-                vm = conn.lookupByID(i)
-                vms.append(vm)
-            except libvirt.libvirtError:
-                # guest probably in process of dieing
-                logging.warn("Failed to lookup domain id %d" % i)
-        # get defined domain
-        names = conn.listDefinedDomains()
-        for name in names:
-            try:
-                vm = conn.lookupByName(name)
-                vms.append(vm)
-            except libvirt.libvirtError:
-                # guest probably in process of dieing
-                logging.warn("Failed to lookup domain name %s" % name)
+        active, inactive = _util.fetch_all_guests(conn)
+        vms = active + inactive
 
         if self.vol_object:
             path = self.vol_object.path()
@@ -705,29 +777,29 @@ class VirtualDisk(VirtualDevice):
         if not path:
             return False
 
+        def count_cb(ctx):
+            c = 0
+
+            c += ctx.xpathEval("count(/domain/devices/disk/source[@dev='%s'])" % path)
+            c += ctx.xpathEval("count(/domain/devices/disk/source[@file='%s'])" % path)
+            return c
+
         count = 0
+        names = []
         for vm in vms:
-            doc = None
-            try:
-                doc = libxml2.parseDoc(vm.XMLDesc(0))
-            except:
-                continue
-            ctx = doc.xpathNewContext()
-            try:
-                try:
-                    count += ctx.xpathEval("count(/domain/devices/disk/source[@dev='%s'])" % path)
-                    count += ctx.xpathEval("count(/domain/devices/disk/source[@file='%s'])" % path)
-                except:
-                    continue
-            finally:
-                if ctx is not None:
-                    ctx.xpathFreeContext()
-                if doc is not None:
-                    doc.freeDoc()
+            xml = vm.XMLDesc(0)
+            tmpcount = _util.get_xml_path(xml, func = count_cb)
+            if tmpcount:
+                count += tmpcount
+                names.append(vm.name())
+
+        ret = False
         if count > 0:
-            return True
-        else:
-            return False
+            ret = True
+        if return_names:
+            ret = names
+
+        return ret
 
     def _get_target_type(self):
         """
