@@ -25,13 +25,30 @@ def _virtinst(msg):
     return gettext.dgettext(gettext_app, msg)
 
 import util
-from Guest import Guest, VirtualNetworkInterface, XenGuest, \
-                  XenNetworkInterface, VirtualGraphics, VirtualAudio
+import Storage
+from Guest import Guest, XenGuest
+from VirtualNetworkInterface import VirtualNetworkInterface, \
+                                    XenNetworkInterface
+from VirtualGraphics import VirtualGraphics
+from VirtualAudio import VirtualAudio
 from VirtualDisk import VirtualDisk, XenDisk
+from VirtualHostDevice import (VirtualHostDevice, VirtualHostDeviceUSB,
+                               VirtualHostDevicePCI)
 from FullVirtGuest import FullVirtGuest
 from ParaVirtGuest import ParaVirtGuest
-from DistroManager import DistroInstaller, PXEInstaller
+from DistroInstaller import DistroInstaller
+from PXEInstaller import PXEInstaller
 from LiveCDInstaller import LiveCDInstaller
+from ImportInstaller import ImportInstaller
 from ImageManager import ImageInstaller
 from CloneManager import CloneDesign
 from User import User
+
+# This represents the PUBLIC API. Any changes to these classes (or 'util.py')
+# must be mindful of this fact.
+__all__ = ["Guest", "XenGuest", "VirtualNetworkInterface",
+           "XenNetworkInterface", "VirtualGraphics", "VirtualAudio",
+           "VirtualDisk", "XenDisk", "FullVirtGuest", "ParaVirtGuest",
+           "DistroInstaller", "PXEInstaller", "LiveCDInstaller",
+           "ImportInstaller", "ImageInstaller", "CloneDesign", "Storage",
+           "User", "util"]
