@@ -27,6 +27,7 @@ from optparse import OptionValueError, OptionParser
 
 import libvirt
 import _util
+import virtinst
 from virtinst import CapabilitiesParser, VirtualNetworkInterface, \
                      VirtualGraphics, VirtualAudio, User
 from virtinst import _virtinst as _
@@ -122,7 +123,7 @@ def getConnection(connect):
     if connect is None:
         fail(_("Could not find usable default libvirt connection."))
 
-    logging.debug("Using libvirt URI connect '%s'" % connect)
+    logging.debug("Using libvirt URI '%s'" % connect)
     return libvirt.open(connect)
 
 #
@@ -360,6 +361,15 @@ def get_sound(sound, guest):
     if sound:
         guest.sound_devs.append(VirtualAudio(model="es1370"))
 
+def get_hostdevs(hostdevs, guest):
+    if not hostdevs:
+        return
+
+    for devname in hostdevs:
+        dev = virtinst.VirtualHostDevice.device_from_node(conn=guest.conn,
+                                                          name=devname)
+        guest.hostdevs.append(dev)
+
 ### Option parsing
 def check_before_store(option, opt_str, value, parser):
     if len(value) == 0:
