@@ -41,7 +41,6 @@ from virtinst import _virtinst as _
 import CapabilitiesParser
 from User import User
 
-
 KEYBOARD_DIR = "/etc/sysconfig/keyboard"
 XORG_CONF = "/etc/X11/xorg.conf"
 
@@ -361,7 +360,9 @@ def default_keymap():
 
     # Set keymap to same as hosts
     import keytable
-    keymap = "en-us"
+    default = "en-us"
+    keymap = None
+
     kt = None
     try:
         f = open(KEYBOARD_DIR, "r")
@@ -385,10 +386,37 @@ def default_keymap():
                 kt = s.split(delim)[1].strip()
         f.close()
 
-    if kt and keytable.keytable.has_key(kt.lower()):
+    if kt == None:
+        logging.debug("Did not parse any usable keymapping.")
+        return default
+
+    kt = kt.lower()
+
+    # Try a simple lookup in the keytable
+    if keytable.keytable.has_key(kt.lower()):
         keymap = keytable.keytable[kt]
     else:
-        logging.debug("Didn't find keymap '%s' in keytable!" % kt)
+        # Try a more intelligent lookup: strip out all '-' and '_', sort
+        # the keytable keys putting the longest first, then compare
+        # by string prefix
+        def len_cmp(a, b):
+            return len(b) - len(a)
+
+        clean_kt = kt.replace("-", "").replace("_", "")
+        sorted_keys = sorted(keytable.keytable.keys(), len_cmp)
+
+        for key in sorted_keys:
+            origkey = key
+            key = key.replace("-", "").replace("_","")
+
+            if clean_kt.startswith(key):
+                keymap = keytable.keytable[origkey]
+                break
+
+    if not keymap:
+        logging.debug("Didn't match keymap '%s' in keytable!" % kt)
+        return default
+
     return keymap
 
 def pygrub_path(conn=None):
@@ -511,23 +539,33 @@ def is_storage_capable(conn):
             return False
     return True
 
-def get_xml_path(xml, path):
-    """return the xpath from the passed xml"""
+def get_xml_path(xml, path=None, func=None):
+    """
+    Return the content from the passed xml xpath, or return the result
+    of a passed function (receives xpathContext as its only arg)
+    """
     doc = None
     ctx = None
     result = None
+
     try:
         doc = libxml2.parseDoc(xml)
         ctx = doc.xpathNewContext()
-        ret = ctx.xpathEval(path)
-        val = None
-        if ret != None:
-            if type(ret) == list:
-                if len(ret) == 1:
-                    val = ret[0].content
-            else:
-                val = ret
-        result = val
+
+        if path:
+            ret = ctx.xpathEval(path)
+            if ret != None:
+                if type(ret) == list:
+                    if len(ret) == 1:
+                        result = ret[0].content
+                else:
+                    result = ret
+
+        elif func:
+            result = func(ctx)
+
+        else:
+            raise ValueError(_("'path' or 'func' is required."))
     finally:
         if doc:
             doc.freeDoc()
